#include  "CGUInit.h"

/**************************************************************************************************************************
	Function name	:   CGU_Init
    Description		:   This function initializes the CGU to configure different clocks in system.

	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCguNo		 	     |  Instance of CGU         				| 0, 1
						uiClkinsel		     |  CLKIN source for CGU    				| 0, 1
						*pClocks		     |  Pointer to Structure which holds        | MSEL, DF, CSEL, SYSSEL, S0SEL,
                                                different Clock multipliers and           S1SEL, DSEL, OSEL values
                                                dividers
                        uiClkoutsel          |  Selects one of the clock to CLKOUT      | Processor dependent
	Return value	:   Success/Error code.
**************************************************************************************************************************/
uint32_t CGU_Init(uint32_t uiCguNo, uint32_t uiClkinsel, CGU_Settings *pClocks,  uint32_t uiClkoutsel)
{
      uint32_t result = 0;
#ifdef __ADSPBF707__
    /* Check if the multipliers do not result in out of range frequencies */
    CGU_Freq_Check_Round(pClocks, 25);                  // Second paramenter is CLKIN value. It is ignored inside this function and is taken from CGUInit.h File

    uint32_t dActionFlags = BITM_ROM_SYSCTRL_CGU_CTL | BITM_ROM_SYSCTRL_CGU_DIV ;
	CGU0_write(dActionFlags, pClocks, uiClkoutsel);

//    result = cgu_write(&Cgu_Config);                  /* Call the ROM API to configure the CGU */
    return result;
#endif

#ifdef __ADSPSC5xx__
    /* Check if the multipliers do not result in out of range frequencies */
    if(uiCguNo == 1)
    {
        *pREG_CDU0_CLKINSEL =   uiClkinsel << BITP_CDU_CLKINSEL_CGU1;
    }

 //   CGU_Freq_Check_Round(pClocks, 25);                  // Second paramenter is CLKIN value. It is ignored inside this function and is taken from CGUInit.h File

    uint32_t dActionFlags = BITM_ROM_SYSCTRL_CGU_CTL | BITM_ROM_SYSCTRL_CGU_DIV ;

     if(uiCguNo == 0)
     {
        if(*pREG_CGU0_STAT & BITM_CGU_STAT_PLLBP)   // Check if processor is in Active mode
        {
            Active_To_Fullon(0);
        }
        CGU0_write(dActionFlags, pClocks, uiClkoutsel);
     }
     else if(uiCguNo == 1)
     {
        if(*pREG_CGU1_STAT & BITM_CGU_STAT_PLLBP)   // Check if processor is in Active mode
        {
            Active_To_Fullon(1);
        }
        CGU1_write(dActionFlags, pClocks, uiClkoutsel);
     }
    return result;
#endif
}


/* @brief       Performs a CGU write operation.
* @details     This function performs the write operation to configure new frequencies as requested
*  			by the user
* @param [in]  dActionFlags     Specifies the action requested by the user
* @param [in]  *pSyscontrol     Pointer to the ROM structure
* @param [in]  *reserved        Reserved for future use
* @return       Status
*				- #0			 													Success
* 				- #BITM_ROM_SYSCTRL_READ_ERR | BITM_ROM_SYSCTRL_CGU_STAT_ERR		Error due to PLL disabled
*				- #BITM_CGU_STAT_WDIVERR | BITM_CGU_STAT_WDFMSERR                   CGU status error occured
*/
uint32_t CGU0_write(uint32_t dActionFlags, CGU_Settings *pCGU_Settings, uint32_t uiClkOutSel)
{
	uint32_t dNewCguDiv;																							/* variable to hold the new CGU-DIV value */
	uint8_t dPllBypassed  = (*pREG_CGU0_STAT & BITM_CGU_STAT_PLLBP);												/* Variable to hold the status of PLL Bypass */
	uint8_t dPllEnabled   = (*pREG_CGU0_STAT & BITM_CGU_STAT_PLLEN);												/* Variable to hold the status of PLL Enable */

	/* Check if write to CGU_DIV register is requested */
	if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)
	{
		dNewCguDiv = 	((OSEL(pCGU_Settings->div_OSEL)) 		|
						(SYSSEL(pCGU_Settings->div_SYSSEL )) 	|
						(CSEL(pCGU_Settings->div_CSEL )) 		|
						(S0SEL(pCGU_Settings->div_S0SEL)) 		|
						(S1SEL(pCGU_Settings->div_S1SEL)) 		|
						(DSEL(pCGU_Settings->div_DSEL )))		&
						(~BITM_CGU_DIV_LOCK);

		if(!dPllBypassed)																							/* Check if PLL is not bypassed */
		{
			dNewCguDiv 	|= (BITM_CGU_DIV_UPDT)  ;																	/* Set the Align and Update bits */
		}

		if( !(dActionFlags & BITM_ROM_SYSCTRL_CGU_AUTO_DIS))														/* Check if disabling the auto handling of ALGN and UPDT fields is requested */
		{
			if(!dPllBypassed)
			{
				dNewCguDiv 	|= (BITM_CGU_DIV_ALGN | BITM_CGU_DIV_UPDT)  ;											/* Set the Align and Update bits */
			}

			if( ((dNewCguDiv) & (DIV_MASK)) == ((*pREG_CGU0_DIV) & (DIV_MASK)) )									/* Check if new values involve a frequecy change */
			{
				dActionFlags 	&=  (~BITM_ROM_SYSCTRL_CGU_DIV);													/* Clear the Flag for write to DIV register */
			}
		}
	}

	if( dActionFlags & BITM_ROM_SYSCTRL_CGU_CTL)																	/* Check if write to CGU_CTL register is requested */
	{
		uint32_t dNewCguCtl  = (MSEL( pCGU_Settings->ctl_MSEL )| DF( pCGU_Settings->ctl_DF ) );

		if( ((dNewCguCtl) & (CTL_MASK)) == ((*pREG_CGU0_CTL) & (CTL_MASK)) )										/* Check if new values involve a frequecy change */
		{
			dActionFlags 	&=  (~BITM_ROM_SYSCTRL_CGU_CTL);														/* Clear the Flag for write to CTL register */
		}
	}

	Program_Cgu0(dActionFlags, dPllBypassed, dPllEnabled, dNewCguDiv, pCGU_Settings);						/* Program the CTL and DIV registers */

	*pREG_CGU0_CLKOUTSEL  = uiClkOutSel;

	return 0;
}

#ifdef __ADSPSC5xx__
uint32_t CGU1_write(uint32_t dActionFlags, CGU_Settings *pCGU_Settings, uint32_t uiClkOutSel)
{
	uint32_t dNewCguDiv;																							/* variable to hold the new CGU-DIV value */
	uint8_t dPllBypassed  = (*pREG_CGU1_STAT & BITM_CGU_STAT_PLLBP);												/* Variable to hold the status of PLL Bypass */
	uint8_t dPllEnabled   = (*pREG_CGU1_STAT & BITM_CGU_STAT_PLLEN);												/* Variable to hold the status of PLL Enable */

	/* Check if write to CGU_DIV register is requested */
	if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)
	{
		dNewCguDiv = 	((OSEL(pCGU_Settings->div_OSEL)) 		|
						(SYSSEL(pCGU_Settings->div_SYSSEL )) 	|
						(CSEL(pCGU_Settings->div_CSEL )) 		|
						(S0SEL(pCGU_Settings->div_S0SEL)) 		|
						(S1SEL(pCGU_Settings->div_S1SEL)) 		|
						(DSEL(pCGU_Settings->div_DSEL )))		&
						(~BITM_CGU_DIV_LOCK);

		if(!dPllBypassed)																							/* Check if PLL is not bypassed */
		{
			dNewCguDiv 	|= (BITM_CGU_DIV_UPDT)  ;																	/* Set the Align and Update bits */
		}

		if( !(dActionFlags & BITM_ROM_SYSCTRL_CGU_AUTO_DIS))														/* Check if disabling the auto handling of ALGN and UPDT fields is requested */
		{
			if(!dPllBypassed)
			{
				dNewCguDiv 	|= (BITM_CGU_DIV_ALGN | BITM_CGU_DIV_UPDT)  ;											/* Set the Align and Update bits */
			}

			if( ((dNewCguDiv) & (DIV_MASK)) == ((*pREG_CGU1_DIV) & (DIV_MASK)) )									/* Check if new values involve a frequecy change */
			{
				dActionFlags 	&=  (~BITM_ROM_SYSCTRL_CGU_DIV);													/* Clear the Flag for write to DIV register */
			}
		}
	}

	if( dActionFlags & BITM_ROM_SYSCTRL_CGU_CTL)																	/* Check if write to CGU_CTL register is requested */
	{
		uint32_t dNewCguCtl  = (MSEL( pCGU_Settings->ctl_MSEL )| DF( pCGU_Settings->ctl_DF ) );

		if( ((dNewCguCtl) & (CTL_MASK)) == ((*pREG_CGU1_CTL) & (CTL_MASK)) )										/* Check if new values involve a frequecy change */
		{
			dActionFlags 	&=  (~BITM_ROM_SYSCTRL_CGU_CTL);														/* Clear the Flag for write to CTL register */
		}
	}

	Program_Cgu1(dActionFlags, dPllBypassed, dPllEnabled, dNewCguDiv, pCGU_Settings);						/* Program the CTL and DIV registers */
	//*pREG_CGU0_CLKOUTSEL  = uiClkOutSel;                                                                          /*  CGU1 Clockoutsel doesnot contro anything*/
	return 0;
}
#endif


/**
* @brief       Program the CGU
* @details     This function the CGU_CTL and CGU_CTL registers
* @param [in]  dActionFlags     Specifies the action requested by the user
* @param [in]  *pSyscontrol     Pointer to the ROM structure
* @param [in]  *reserved        Reserved for future use
* @return       None
*/
void Program_Cgu0(uint32_t dActionFlags, uint32_t dPllBypassed, uint32_t dPllEnabled, uint32_t dNewCguDiv, CGU_Settings *pCGU_Settings)
{
	if( !(dActionFlags & BITM_ROM_SYSCTRL_CGU_CTL) )																/* Check if CGU_CTL write is not requested */
	{
		if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)																/* Check if CGU_DIV needs to be written */
		{
			if( (!dPllBypassed) && (dPllEnabled) )																	/* Check if PLL is not bypassed or disabled */
			{
				while(*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN);      												/* Check if clocks are aligned before modyfying the DIV register */
				/* !!!!!! No Status bit to know error condition  !!!!!  */
				/*  !!!!!! Decide if this loop needs to be broken !!!!!  */
			}

			*pREG_CGU0_DIV = dNewCguDiv;																			/* Update the new Divider values */

			if(  (!dPllBypassed) && (dPllEnabled) )																	/* Check if PLL is not bypassed or disabled */
			{
				while(*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN);													/* Wait till clocks are aligned */
			}
		}
	}/* end of CGU_DIV Write */

	else																											/* Update both CGU_CTL and CGU_DIV registers */
	{
		if(  (!dPllBypassed) && (dPllEnabled) )																		/* Check if PLL is not bypassed or disabled */
		{
			while( !((*pREG_CGU0_STAT & (BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_CLKSALGN) ) ==((BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK) & (~BITM_CGU_STAT_CLKSALGN))));
		} 																											/* Check if PLL is enabled , PLL is not locking and Clocks are aligned */

		if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)
		{
			*pREG_CGU0_DIV =  dNewCguDiv & (~BITM_CGU_DIV_ALGN) & (~BITM_CGU_DIV_UPDT);								/* Clear the Align and Update bit, since CTL is also written */
		}
		*pREG_CGU0_CTL =  (MSEL( pCGU_Settings->ctl_MSEL ) | DF( pCGU_Settings->ctl_DF ) ) & (~BITM_CGU_CTL_LOCK);


		if(  (!dPllBypassed) && (dPllEnabled) )																		/* Check if PLL is not bypassed or disabled */
		{
			while( !((*pREG_CGU0_STAT & (BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_CLKSALGN) ) == (BITM_CGU_STAT_PLOCK & (~BITM_CGU_STAT_CLKSALGN))));
		} 																											/* Wait till PLL is locked and clocks are aligned */
	}/* end of CGU_DIV and CGU_CTL Write */
}

#ifdef __ADSPSC5xx__
void Program_Cgu1(uint32_t dActionFlags, uint32_t dPllBypassed, uint32_t dPllEnabled, uint32_t dNewCguDiv, CGU_Settings *pCGU_Settings)
{
	if( !(dActionFlags & BITM_ROM_SYSCTRL_CGU_CTL) )																/* Check if CGU_CTL write is not requested */
	{
		if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)																/* Check if CGU_DIV needs to be written */
		{
			if( (!dPllBypassed) && (dPllEnabled) )																	/* Check if PLL is not bypassed or disabled */
			{
				while(*pREG_CGU1_STAT & BITM_CGU_STAT_CLKSALGN);      												/* Check if clocks are aligned before modyfying the DIV register */
				/* !!!!!! No Status bit to know error condition  !!!!!  */
				/*  !!!!!! Decide if this loop needs to be broken !!!!!  */
			}

			*pREG_CGU1_DIV = dNewCguDiv;																			/* Update the new Divider values */

			if(  (!dPllBypassed) && (dPllEnabled) )																	/* Check if PLL is not bypassed or disabled */
			{
				while(*pREG_CGU1_STAT & BITM_CGU_STAT_CLKSALGN);													/* Wait till clocks are aligned */
			}
		}
	}/* end of CGU_DIV Write */

	else																											/* Update both CGU_CTL and CGU_DIV registers */
	{
		if(  (!dPllBypassed) && (dPllEnabled) )																		/* Check if PLL is not bypassed or disabled */
		{
			while( !((*pREG_CGU1_STAT & (BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_CLKSALGN) ) ==((BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK) & (~BITM_CGU_STAT_CLKSALGN))));
		} 																											/* Check if PLL is enabled , PLL is not locking and Clocks are aligned */

		if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)
		{
			*pREG_CGU1_DIV =  dNewCguDiv & (~BITM_CGU_DIV_ALGN) & (~BITM_CGU_DIV_UPDT);								/* Clear the Align and Update bit, since CTL is also written */
		}
		*pREG_CGU1_CTL =  (MSEL( pCGU_Settings->ctl_MSEL ) | DF( pCGU_Settings->ctl_DF ) ) & (~BITM_CGU_CTL_LOCK);


		if(  (!dPllBypassed) && (dPllEnabled) )																		/* Check if PLL is not bypassed or disabled */
		{
			while( !((*pREG_CGU1_STAT & (BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_CLKSALGN) ) == (BITM_CGU_STAT_PLOCK & (~BITM_CGU_STAT_CLKSALGN))));
		} 																											/* Wait till PLL is locked and clocks are aligned */
	}/* end of CGU_DIV and CGU_CTL Write */
}
#endif


/**************************************************************************************************************************
	Function name	:   CDU_Init
    Description		:   This function allows the user to map the different clocks from CGU0 and CGU1 to different targets

	Arguments		:	Parameter			 |	Description		 						| Valid values
						*pClock_Map		     |  Pointer to Structure which              | CLKO0 - CLKO9
                                                which holds the CDU mapping values
                                                and enable bits

                          -----------------------------------------------------------------------------------
                          | Structure value     0           1           2           3                                    -----------------------------------------------------------------------------------
                                CLKO0	        CCLK0_0	    SYSCLK0_0	NA	        NA	    	
                                CLKO1	        CCLK0_0	    SYSCLK0_0	NA	        NA	    	
                                CLKO2	        CCLK1_0	    SYSCLK0_0	NA	        NA	    	
                                CLKO3	        DCLK_0	    DCLK_1	    NA	        NA	    	
                                CLKO4	        OCLK_0	    OCLK_1	    DCLK_1	    NA	    	
                                CLKO5	        OCLK_0	    OCLK_1	    DCLK_1	    DCLK_0	   	
                                CLKO6	        OCLK_0	    CCLK0_1	    NA	        NA	    	
                                CLKO7	        SCLK1_0	    SCLK1_1	    CCLK0_1	    OCLK_0	   	
                                CLKO8	        SCLK0_0	    SCLK0_1	    CCLK1_1	    DCLK_1	   	
                                CLKO9	        OCLK_0/2	CCLK1_1/2	CCLK1_1	    DCLK_1		

	Return value	:   None.
**************************************************************************************************************************/
#ifdef __ADSPSC5xx__
void CDU_Init(_CDU_MAPPINGS *pClock_Map)
{
    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO0);							// wait while configuration change is in progress
    *pREG_CDU0_CFG0    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG0     = (pClock_Map->cfg0_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg0_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO0
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO0);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO1);							// wait while configuration change is in progress
    *pREG_CDU0_CFG1    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG1     = (pClock_Map->cfg1_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg1_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO1
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO1);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO2);							// wait while configuration change is in progress
    *pREG_CDU0_CFG2    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG2     = (pClock_Map->cfg2_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg2_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO2
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO2);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO3);							// wait while configuration change is in progress
    *pREG_CDU0_CFG3    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG3     = (pClock_Map->cfg3_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg3_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO3
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO3);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO4);							// wait while configuration change is in progress
    *pREG_CDU0_CFG4    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG4     = (pClock_Map->cfg4_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg4_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO4
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO4);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO5);							// wait while configuration change is in progress
    *pREG_CDU0_CFG5    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG5     = (pClock_Map->cfg5_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg5_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO5
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO5);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO6);							// wait while configuration change is in progress
    *pREG_CDU0_CFG6    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG6     = (pClock_Map->cfg6_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg6_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO6
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO6);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO7);							// wait while configuration change is in progress
    *pREG_CDU0_CFG7    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG7     = (pClock_Map->cfg7_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg7_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO7
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO7);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO8);							// wait while configuration change is in progress
    *pREG_CDU0_CFG8    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG8     = (pClock_Map->cfg8_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg8_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO8
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO8);							// wait while configuration change is in progress


    while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO9);							// wait while configuration change is in progress
    *pREG_CDU0_CFG9    &= ~BITM_CDU_CFG_LOCK;                               // unlock the MMR
	*pREG_CDU0_CFG9     = (pClock_Map->cfg9_SEL << BITP_CDU_CFG_SEL) | pClock_Map->cfg9_EN | BITM_CDU_CFG_LOCK;		// Change the source of CLKO9
	while(*pREG_CDU0_STAT & BITM_CDU_STAT_CLKO9);							// wait while configuration change is in progress

}
#endif


/**************************************************************************************************************************
	Function name	:   CGU_Freq_Check_Round
    Description		:   This function check round the user frequencies if they are out of spec

	Arguments		:	Parameter			 |	Description		 						| Valid values
                        *pClocks		     |  Pointer to Structure which holds        | MSEL, DF, CSEL, SYSSEL, S0SEL,
                                                different Clock multipliers and           S1SEL, DSEL, OSEL values
                                                dividers

	Return value	:   None.
**************************************************************************************************************************/
void CGU_Freq_Check_Round(CGU_Settings *pClocks, float uiClkinValue)
{
#ifdef __ADSPBF707__
    // Calculte the CLKIN locally
    float uiClkinVal   = (float)CLKIN_BF70x/1000000;

    uint32_t uiPllclk_Max   = PLLCLK_MAX_70X;
    uint32_t uiCclk_Max     = CCLK_MAX_70X;
    uint32_t uiSysclk_Max   = SYSCLK_MAX_70X;
    uint32_t uiS0clk_Max    = S0CLK_MAX_70X;
    uint32_t uiS1clk_Max    = S1CLK_MAX_70X;
    uint32_t uiDclk_Max     = DDRCLK_MAX_70X;
#endif

#ifdef __ADSPSC5xx__
    // Calculte the CLKIN locally
    float uiClkinVal   = (float)CLKIN_SC573/1000000;

    uint32_t uiPllclk_Max   = PLLCLK_MAX_573;
    uint32_t uiCclk_Max     = CCLK_MAX_573;
    uint32_t uiSysclk_Max   = SYSCLK_MAX_573;
    uint32_t uiS0clk_Max    = S0CLK_MAX_573;
    uint32_t uiS1clk_Max    = S1CLK_MAX_573;
    uint32_t uiDclk_Max     = DDRCLK_MAX_573;
#endif


    float Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
    float Core_Clock   = Pll_Clock/pClocks->div_CSEL;
    float Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
    float S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
    float S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
    float Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
    float Out_Clock    = Pll_Clock/pClocks->div_OSEL;

    if( Pll_Clock > uiPllclk_Max)                                  /* If PLL clock is out of spec reduce MSEL to make PLL clcok within the spec */
    {
        while(Pll_Clock > uiPllclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
        }

        // Recalculate all the clocks
        Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
        Core_Clock   = Pll_Clock/pClocks->div_CSEL;
        Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
        S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
        S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
        Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
        Out_Clock    = Pll_Clock/pClocks->div_OSEL;

    }

    if( Core_Clock > uiCclk_Max)                         /* If Core clock is out of spec reduce MSEL to make Core clock within the spec */
    {
        while(Core_Clock > uiCclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            Core_Clock   = Pll_Clock/pClocks->div_CSEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }

    if( Sys_Clock > uiSysclk_Max)                         /* If Sys clock is out of spec reduce MSEL to make Sys clock within the spec */
    {
        while(Sys_Clock > uiSysclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }

    if( S0_Clock > uiS0clk_Max)                         /* If S0 clock is out of spec reduce MSEL to make S0 clock within the spec */
    {
        while(S0_Clock > uiS0clk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }


    if( S1_Clock > uiS1clk_Max)                         /* If S1 clock is out of spec reduce MSEL to make S0 clock within the spec */
    {
        while(S1_Clock > uiS1clk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }

    if( Ddr_Clock > uiDclk_Max)                         /* If Ddr_Clock is out of spec reduce MSEL to make DDR clock within the spec */
    {
        while(Ddr_Clock > uiDclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }
}


/**************************************************************************************************************************
	Function name	:   Fullon_To_Active
    Description		:   This function changes the Operating mode from FullOn to Active
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
void Fullon_To_Active(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
    	uint32_t oldCGU0_STAT ;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT & BITM_CGU_STAT_CLKSALGN))
            {   break; }
        }

	    *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLBPST;      // Enable PLL ByPass

	    while(1)                                        // Wait loop to ensure post-switch exit conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if (oldCGU0_STAT & BITM_CGU_STAT_PLLBP)
            {   break; }
        }
    }
#ifdef __ADSPSC5xx__
     if(uiCGU_ID == 1)
    {
    	uint32_t oldCGU0_STAT1 ;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if ((oldCGU0_STAT1 & BITM_CGU_STAT_PLLEN) && (oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT1 & BITM_CGU_STAT_CLKSALGN))
            {   break; }
        }

	    *pREG_CGU1_PLLCTL = BITM_CGU_PLLCTL_PLLBPST;   // Enable PLL ByPass

	    while(1)                                    // Wait loop to ensure post-switch exit conditions
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if ((oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP))
            {   break; }
        }
    }
#endif
}

/**************************************************************************************************************************
	Function name	:   Active_To_Fullon
    Description		:   This function changes the Operating mode from Active to FullOn
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
void Active_To_Fullon(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
        uint32_t oldCGU0_STAT;

        while(1)                                        // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && (oldCGU0_STAT & BITM_CGU_STAT_PLLBP))
            {   break; }
        }

        *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLBPCL;

        while(1)                                        // Wait Loop to ensure post-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if ((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT & BITM_CGU_STAT_CLKSALGN))
            {   break; }
        }
    }
#ifdef __ADSPSC5xx__
    if(uiCGU_ID == 1)
    {
        uint32_t oldCGU0_STAT1;

        while(1)                                        // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if((oldCGU0_STAT1 & BITM_CGU_STAT_PLLEN) && (oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP))
            {   break; }
        }

        *pREG_CGU1_PLLCTL = BITM_CGU_PLLCTL_PLLBPCL;

        while(1)                                        // Wait Loop to ensure post-switch conditions
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if ((oldCGU0_STAT1 & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT1 & BITM_CGU_STAT_CLKSALGN))
            {   break; }
        }
    }
#endif
}

/**************************************************************************************************************************
	Function name	:   Active_To_ActivePLLDis
    Description		:   This function changes the Operating mode from Active to Active with PLL Disabled
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
void Active_To_ActivePLLDis(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
        uint32_t  oldCGU0_STAT;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && (oldCGU0_STAT & BITM_CGU_STAT_PLLBP))
            {  break;  }
        }

        *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLDIS;

        while(1)                                    // Wait Loop to ensure post-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if(~(oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT & BITM_CGU_STAT_CLKSALGN) && (oldCGU0_STAT & BITM_CGU_STAT_PLLBP))
            {  break;  }
        }
    }
#ifdef __ADSPSC5xx__
    if(uiCGU_ID == 1)
    {
        uint32_t  oldCGU0_STAT1;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if((oldCGU0_STAT1 & BITM_CGU_STAT_PLLEN) && (oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP))
            {  break;  }
        }

        *pREG_CGU1_PLLCTL = BITM_CGU_PLLCTL_PLLDIS;

        while(1)                                    // Wait Loop to ensure post-switch conditions
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if(~(oldCGU0_STAT1 & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT1 & BITM_CGU_STAT_CLKSALGN) && (oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP))
            {  break;  }
        }
    }
#endif
}

/**************************************************************************************************************************
	Function name	:   ActivePLLDis_To_Active
    Description		:   This function changes the Operating mode from Active with PLL Disabled to Active
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
void ActivePLLDis_To_Active(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
    	uint32_t  oldCGU0_STAT;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT & BITM_CGU_STAT_PLLEN))
            {  break;   }
        }

        *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLEN;

        while (1)
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && (oldCGU0_STAT & BITM_CGU_STAT_PLLEN))
            {  break;   }
        }
    }
#ifdef __ADSPSC5xx__
    if(uiCGU_ID == 1)
    {
    	uint32_t  oldCGU0_STAT1;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if((oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT1 & BITM_CGU_STAT_PLLEN))
            {  break;   }
        }

        *pREG_CGU1_PLLCTL = BITM_CGU_PLLCTL_PLLEN;

        while (1)
        {
            oldCGU0_STAT1 = *pREG_CGU1_STAT;
            if((oldCGU0_STAT1 & BITM_CGU_STAT_PLLBP) && (oldCGU0_STAT1 & BITM_CGU_STAT_PLLEN))
            {  break;   }
        }
    }
#endif
}
